package ufrn.imd.controller.impl;

import lombok.extern.java.Log;
import ufrn.imd.controller.Controller;
import ufrn.imd.domain.Client;
import ufrn.imd.service.impl.DepositServiceImpl;
import ufrn.imd.utils.ServerResponse;
import ufrn.imd.utils.message.DepositMessage;

import java.net.MalformedURLException;
import java.rmi.AlreadyBoundException;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.server.UnicastRemoteObject;
import java.util.*;

import static ufrn.imd.utils.NameSpace.HOST_DEPOSIT_SERVICE;
import static ufrn.imd.utils.NameSpace.PORT_DEPOSIT_SERVICE;

@Log
public class DepositControllerImpl extends UnicastRemoteObject implements Controller {

    private volatile Queue<Client> clients = new ArrayDeque<>();
    private final DepositServiceImpl service;

    private volatile Queue<ServerResponse> serverResponses = new ArrayDeque<>();
    public DepositControllerImpl(DepositServiceImpl service) throws RemoteException, MalformedURLException, AlreadyBoundException {
        super();
        log.info("Starting Deposit service!");

        log.info(String.format("Initializing server in %s", HOST_DEPOSIT_SERVICE));
        this.service = service;
        LocateRegistry.createRegistry(PORT_DEPOSIT_SERVICE);
        Naming.bind(HOST_DEPOSIT_SERVICE, this);
    }

    public ServerResponse getResponse(Object obj) throws RemoteException {
        DepositMessage depositMessage = (DepositMessage)obj;
        return this.service.deposit(depositMessage.getValue(), Optional.of(depositMessage.getClient()));
    }

}
